/*
 * Planet namer
 * Make planet.nm file from (randomised) source list
 * by Roger Burton West <roger@firedrake.demon.co.uk>
 * Converted from ReXX to C
 * GNUWare
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#define MAXNAMES 2000 /* feel free to change this one */
#define MAXPLAN 500 /* intrinsic to VGAP */

int main(int argc,char *argv[]) {
  char srcfile[60], plnmfile[80];
  char *planname[MAXNAMES];
  char *out[MAXPLAN];
  char *planbase;
  char inp[80],work[80];
  char *c;
  FILE *ifh;
  FILE *ofh;
  int k,x,t,l,pn;
  srcfile[0]=NULL;
  plnmfile[0]=NULL;
  switch (argc) {
    case 1:
      break;
    case 2:
      strcpy(srcfile,argv[1]);
      break;
    default:
      strcpy(srcfile,argv[1]);
      strcpy(plnmfile,argv[2]);
      break;
  }

/* input parameter checking */
  if (srcfile[0]==0) {
    printf("This tool will copy planet names to PLANET.NM in the correct format.\n");
    printf("Usage: PLANNAME srcfile {dir}\n");
    printf("Caution: any existing planet.nm will be overwritten!\n");
    printf("Input file should be ASCII, one name per line;\n");
    printf(" blank lines and lines with ' as first character will be skipped.\n");
    printf(" as will lines with >20 characters.\n");
    printf("Name order will be randomised, but only the first 2,000.\n");
    printf(" lines of <srcfile> will be read.\n");
    printf("Special case: lines of format 'number:name' will be interpreted\n");
    printf(" as a directive to assign <name> to planet <number>.\n");
    return(1);
  }
  fprintf(stderr,"PLANNAME v0.1 by Roger Burton West <rburtonw@nyx.net>\n");
  planname[0]=malloc(MAXNAMES*21);
  if (planname[0]==NULL) {
    fprintf(stderr,"Can't allocate enough memory for planet names\n");
    return(1);
  }
  planbase=planname[0];
  strcpy(planname[0],"Another Planet      ");
  for (k=1;k<MAXNAMES;k++) {
    planname[k]=planname[k-1]+21;
    strcpy(planname[k],"Another Planet      ");
  }
  out[0]=malloc(MAXPLAN*21);
  if (out[0]==NULL) {
    fprintf(stderr,"Can't allocate enough memory for planet names\n");
    return(1);
  }
  *out[0]=NULL;
  for (k=1;k<MAXPLAN;k++) {
    out[k]=out[k-1]+21;
    *out[k]=NULL;
  }
  srandom(time(NULL));
  ifh=fopen(srcfile,"rt");
  if (!ifh) {
    printf("Can't find input file.\n");
    return(2);
  }

  /* tidy dirname, set up files and variables */
  if (plnmfile[0] == NULL) {
    strcpy(plnmfile,".");
  }
  if (plnmfile[strlen(plnmfile)-1] != '\\') {
    strcat(plnmfile,"\\");
  }
  strcat(plnmfile,"planet.nm");
  ofh=fopen(plnmfile,"wb");

  /* read input file */
  k=0;
  x=0;
  c=inp;
  while ((c != NULL) & (k < MAXNAMES)) {
    c=fgets(inp,80,ifh);
    if (c != NULL) {
      inp[strlen(inp)-1]=0;
      pn=0;
      work[0]=NULL;
      for (t=0;t<strlen(inp);t++) {
        if (inp[t] == ':') {
          break;
        }
      }
      if (t < strlen(inp)) {
        c=inp+t+1;
        strcpy(work,c);
        inp[t]=0;
        sscanf(inp,"%d",&pn);
        work[20]=NULL;
        t=20-strlen(work);
        if (t>0) {
          for(;t>0;t--) {
            strcat(work," ");
          }
        }
        strcpy(out[pn-1],work);
        inp[0]=NULL;
        x++;
      }
      if ((inp[0]!='\'') & (inp[0]!=NULL) & (strlen(inp) <= 20)) {
        inp[20]=NULL;
        t=20-strlen(inp);
        if (t>0) {
          for(;t>0;t--) {
            strcat(inp," ");
          }
        }
        strcpy(planname[k++],inp);
      }
    }
  }
  printf("General planet names: %d\n",k);
  if (x>0) printf("Special name assignments: %d\n",x);

  /* bounds check */
  if (k+x<MAXPLAN) {
    printf("Error: insufficient planet names.\n");
    free(planbase);
    free(out[0]);
    exit(1);
  }

  /* write output file */
  for(l=0;l<MAXPLAN;l++) {
    if (*out[l]==NULL) {
      t=random() % k;
      fwrite(planname[t],1,20,ofh);
      planname[t]=planname[k-1];
      k--;
    } else {
      fwrite(out[l],1,20,ofh);
    }
  }

  fclose(ifh);
  fclose(ofh);
  free(planbase);
  free(out[0]);
  return(0);
}
